//! Implementation of the Unreliable, Unordered, Datagram Limited
//! transmission model
//!
//! AKA: UDP-like. "Limited" because the data must fit in a single packet
//!

use crate::connection::stream::{
    Chunk, Error, Sequence, SequenceStart, StreamData,
};

use ::std::collections::{BTreeMap, VecDeque};

#[cfg(test)]
mod tests;

/// UnReliable, UnOrdered, Datagram, Limited to the packet size
/// AKA: UDP-like
#[derive(Debug)]
pub(crate) struct UnreliableUnorderedDatagramLimited {
    received: VecDeque<(SequenceStart, Vec<u8>)>,
}

impl UnreliableUnorderedDatagramLimited {
    pub(crate) fn new() -> Self {
        Self {
            received: VecDeque::with_capacity(4),
        }
    }
    pub(crate) fn get(&mut self) -> (SequenceStart, Vec<u8>) {
        match self.received.pop_front() {
            Some(data) => data,
            None => (SequenceStart(Sequence::min()), Vec::new()),
        }
    }
    pub(crate) fn recv(&mut self, chunk: Chunk) -> Result<StreamData, Error> {
        if !chunk.flag_start || !chunk.flag_end {
            return Err(Error::WrongFlags);
        }
        self.received
            .push_back((SequenceStart(chunk.sequence), chunk.data.to_vec()));
        Ok(StreamData::Ready)
    }
}
